@@ -25,7 +25,7 @@ from bisect import bisect_left
 from pitivi.signalinterface import Signallable
 from pitivi.log.loggable import Loggable
 from pitivi.utils import UNKNOWN_DURATION, closest_item, PropertyChangeTracker
-from pitivi.timeline.track import SourceTrackObject, TrackError
+from pitivi.timeline.track import TrackObject, SourceTrackObject, TrackError
 from pitivi.stream import match_stream_groups_map
 from pitivi.utils import start_insort_right, infinity, getPreviousObject, \
         getNextObject
@@ -36,8 +36,10 @@ SELECT = 0
 """Set the selection to the given set."""
 UNSELECT = 1
 """Remove the given set from the selection."""
-SELECT_ADD = 1
+SELECT_ADD = 2
 """Extend the selection with the given set"""
+SELECT_BETWEEN = 3
+"""Select a range of clips"""
 
 class TimelineError(Exception):
     """Base Exception for errors happening in L{Timeline}s or L{TimelineObject}s"""
@@ -135,7 +137,7 @@ class TimelineObject(Signallable, Loggable):
         @raises TimelineError: If the object doesn't control any C{TrackObject}s.
         """
         if not self.track_objects:
-            raise TimelineError()
+            raise TimelineError("TimelineObject doesn't control any TrackObjects")
 
         if snap:
             position = self.timeline.snapToEdge(position, position + self.duration)
@@ -179,7 +181,7 @@ class TimelineObject(Signallable, Loggable):
         @raises TimelineError: If the object doesn't control any C{TrackObject}s.
         """
         if not self.track_objects:
-            raise TimelineError()
+            raise TimelineError("TimelineObject doesn't control any TrackObjects")
 
         if snap:
             position = self.start + duration
@@ -212,7 +214,7 @@ class TimelineObject(Signallable, Loggable):
         @raises TimelineError: If the object doesn't control any C{TrackObject}s.
         """
         if not self.track_objects:
-            raise TimelineError()
+            raise TimelineError("TimelineObject doesn't control any TrackObjects")
 
         for track_object in self.track_objects:
             track_object.setObjectInPoint(position)
@@ -241,7 +243,7 @@ class TimelineObject(Signallable, Loggable):
         @raises TimelineError: If the object doesn't control any C{TrackObject}s.
         """
         if not self.track_objects:
-            raise TimelineError()
+            raise TimelineError("TimelineObject doesn't control any TrackObjects")
 
         for track_object in self.track_objects:
             track_object.setObjectMediaDuration(position)
@@ -263,7 +265,7 @@ class TimelineObject(Signallable, Loggable):
         @raises TimelineError: If the object doesn't control any C{TrackObject}s.
         """
         if not self.track_objects:
-            raise TimelineError()
+            raise TimelineError("TimelineObject doesn't control any TrackObjects")
 
         for track_object in self.track_objects:
             track_object.setObjectPriority(priority)
@@ -316,7 +318,7 @@ class TimelineObject(Signallable, Loggable):
         @raises TimelineError: If the object doesn't control any C{TrackObject}s.
         """
         if not self.track_objects:
-            raise TimelineError()
+            raise TimelineError("TimelineObject doesn't control any TrackObjects")
 
         if snap:
             position = self.timeline.snapToEdge(position)
@@ -349,7 +351,7 @@ class TimelineObject(Signallable, Loggable):
         it is up to the caller to add the returned 'half' object to a C{Timeline}.
         """
         if not self.track_objects:
-            raise TimelineError()
+            raise TimelineError("TimelineObject doesn't control any TrackObjects")
 
         other = self.copy(copy_track_objects=False)
 
@@ -388,11 +390,11 @@ class TimelineObject(Signallable, Loggable):
         same duration as the other objects controlled.
         """
         if obj.timeline_object is not None:
-            raise TimelineError()
+            raise TimelineError("TrackObject already controlled by another TimelineObject")
 
         if obj in self.track_objects:
             # FIXME : couldn't we just silently return ?
-            raise TimelineError()
+            raise TimelineError("TrackObject already controlled by this TimelineObject")
 
         if self.track_objects:
             # multiple track objects are used for groups.
@@ -413,7 +415,7 @@ class TimelineObject(Signallable, Loggable):
             existing_track_object = self.track_objects[0]
             if obj.start != existing_track_object.start or \
                     obj.duration != existing_track_object.duration:
-                raise TimelineError()
+                raise TimelineError("New TrackObject doesn't have same duration as other controlled TrackObjects")
 
         # FIXME: cycle
         obj.timeline_object = self
@@ -430,13 +432,13 @@ class TimelineObject(Signallable, Loggable):
         @raises TimelineError: If the Track object isn't controlled by this TimelineObject.
         """
         if obj.track is None:
-            raise TimelineError()
+            raise TimelineError("TrackObject doesn't belong to any Track")
 
         try:
             self.track_objects.remove(obj)
             obj.timeline_object = None
         except ValueError:
-            raise TimelineError()
+            raise TimelineError("TrackObject doesn't belong to this TimelineObject")
 
         self.emit("track-object-removed", obj)
 
@@ -457,6 +459,7 @@ class Selection(Signallable):
 
     def __init__(self):
         self.selected = set([])
+        self.last_single_obj = None
 
     def setToObj(self, obj, mode):
         """
@@ -476,11 +479,11 @@ class Selection(Signallable):
         Selection.
         """
         if timeline_object in self.timeline_objects:
-            raise TimelineError()
+            raise TimelineError("TrackObject already in this selection")
 
     # FIXME : it took me 10 mins to understand what this method does... a more obvious
     # name would be better :)
-    def setTo(self, selection, mode):
+    def setTo(self, objs, mode):
         """
         Update the current selection.
 
@@ -494,8 +497,13 @@ class Selection(Signallable):
 
         @see: L{setToObj}
         """
-        # get the L{TrackObject}s for the given TimelineObjects
-        selection = set([obj.timeline_object for obj in selection])
+        # get a list of timeline objects
+        selection = set()
+        for obj in objs:
+            if isinstance(obj, TrackObject):
+                selection.add(obj.timeline_object)
+            else:
+                selection.add(obj)
         old_selection = self.selected
         if mode == SELECT_ADD:
             selection = self.selected | selection
@@ -503,6 +511,9 @@ class Selection(Signallable):
             selection = self.selected - selection
         self.selected = selection
 
+        if len(self.selected) == 1:
+            self.last_single_obj = iter(selection).next()
+
         for obj in self.selected - old_selection:
             obj.selected = True
         for obj in old_selection - self.selected:
@@ -560,10 +571,10 @@ class Link(object):
 
     def addTimelineObject(self, timeline_object):
         if timeline_object.link is not None:
-            raise TimelineError()
+            raise TimelineError("TimelineObject already in a Link")
 
         if timeline_object in self.timeline_objects:
-            raise TimelineError()
+            raise TimelineError("TimelineObject already controlled by this Link")
 
         self.timeline_objects.add(timeline_object)
 
@@ -586,7 +597,7 @@ class Link(object):
         try:
             self.timeline_objects.remove(timeline_object)
         except KeyError:
-            raise TimelineError()
+            raise TimelineError("TimelineObject not controlled by this Link")
 
         tracker = self.property_trackers.pop(timeline_object)
         tracker.disconnectFromObject(timeline_object)
@@ -706,7 +717,7 @@ class TimelineEdges(object):
 
     def addTrackObject(self, track_object):
         if track_object in self.by_object:
-            raise TimelineError()
+            raise TimelineError("TrackObject already controlled by this TimelineEdge")
 
         start = track_object.start
         end = track_object.start + track_object.duration
@@ -724,7 +735,7 @@ class TimelineEdges(object):
         try:
             old_start, old_end = self.by_object.pop(track_object)
         except KeyError:
-            raise TimelineError()
+            raise TimelineError("TrackObject not controlled by this TimelineEdge")
 
         try:
             del self.changed_objects[track_object]
@@ -784,7 +795,7 @@ class TimelineEdges(object):
         index = bisect_left(self.edges, start, lo=lo)
         # check if start is a valid edge
         if index == len(self.edges) or self.edges[index] != start:
-            raise TimelineError()
+            raise TimelineError("Start (%r) is not a valid edge" % start)
 
         del self.edges[index]
         lo = index
@@ -793,7 +804,7 @@ class TimelineEdges(object):
             index = bisect_left(self.edges, end, lo=lo)
             # check if end is a valid edge
             if index == len(self.edges) or self.edges[index] != end:
-                raise TimelineError()
+                raise TimelineError("End (%r) is not a valid edge")
 
             del self.edges[index]
 
@@ -1011,7 +1022,7 @@ class EditingContext(object):
         pass
 
     def _rippleTo(self, position, priority):
-        pass
+        return position, priority
 
     def _finishDefault(self):
         pass
@@ -1037,6 +1048,18 @@ class EditingContext(object):
 
         return position, priority
 
+    def _getGapsAtPriority(self, priority, timeline_objects, tracks=None):
+        gaps = SmallestGapsFinder(timeline_objects)
+        prio_diff = priority - self.focus.priority
+
+        for timeline_object in timeline_objects:
+            left_gap, right_gap = Gap.findAroundObject(timeline_object,
+                    timeline_object.priority + prio_diff, tracks)
+            gaps.update(left_gap, right_gap)
+
+        return gaps.left_gap, gaps.right_gap
+
+
 class MoveContext(EditingContext):
 
     """An editing context which sets the start point of the editing targets.
@@ -1045,15 +1068,26 @@ class MoveContext(EditingContext):
     def __init__(self, timeline, focus, other):
         EditingContext.__init__(self, timeline, focus, other)
 
-        self.timeline_objects = set(track_object.timeline_object
-                for track_object in other)
-        self.timeline_objects.add(focus.timeline_object)
-
         min_priority = infinity
         earliest = infinity
         latest = 0
         self.default_originals = {}
-        for timeline_object in self.timeline_objects:
+        self.timeline_objects = set([])
+        self.tracks = set([])
+        all_objects = set(other)
+        all_objects.add(focus)
+        for obj in all_objects:
+            if isinstance(obj, TrackObject):
+                timeline_object = obj.timeline_object
+                self.tracks.add(obj.track)
+            else:
+                timeline_object = obj
+                timeline_object_tracks = set(track_object.track for track_object
+                        in timeline_object.track_objects)
+                self.tracks.update(timeline_object_tracks)
+
+            self.timeline_objects.add(timeline_object)
+
             self.default_originals[timeline_object] = \
                     self._getTimelineObjectValues(timeline_object)
 
@@ -1066,6 +1100,7 @@ class MoveContext(EditingContext):
                 self.timeline_objects)
 
         self.min_priority = focus.priority - min_priority
+        self.min_position = focus.start - earliest
 
         # get the span over all clips for edge snapping
         self.default_span = latest - earliest
@@ -1091,14 +1126,9 @@ class MoveContext(EditingContext):
             timeline_objects = self.timeline_objects_plus_ripple
         else:
             timeline_objects = self.timeline_objects
-        gaps = SmallestGapsFinder(timeline_objects)
-        prio_diff = priority - self.focus.priority
 
-        for timeline_object in timeline_objects:
-            gaps.update(*Gap.findAroundObject(timeline_object,
-                    timeline_object.priority + prio_diff))
-
-        return gaps.left_gap, gaps.right_gap
+        return EditingContext._getGapsAtPriority(self,
+                priority, timeline_objects, self.tracks)
 
     def setMode(self, mode):
         if mode == self.ROLL:
@@ -1108,27 +1138,123 @@ class MoveContext(EditingContext):
     def _finishDefault(self):
         self._restoreValues(self.default_originals)
 
+    def _overlapsAreTransitions(self, focus, priority):
+        left_gap, right_gap = Gap.findAroundObject(focus)
+
+        focus_end = focus.start + focus.duration
+
+        # left_transition
+        if left_gap.duration < 0:
+
+            left_obj = left_gap.left_object
+            left_end = left_obj.start + left_obj.duration
+
+            if left_end > focus_end:
+                return False
+
+            # check that the previous previous object doesn't end after
+            # our start time. we shouldn't have to go back further than
+            # this because
+            #   * we are only moving one object
+            #   * if there was more than one clip overlaping the previous
+            #   clip, it too would be invalid, since that clip would
+            #   overlap the previous and previous previous clips
+
+            try:
+                prev_prev = self.timeline.getPreviousTimelineObject(left_obj)
+                if prev_prev.start + prev_prev.duration > self.focus.start:
+                    return False
+            except TimelineError:
+                pass
+
+        # right transition
+        if right_gap.duration < 0:
+
+            right_obj = right_gap.right_object
+            right_end = right_obj.start + right_obj.duration
+
+            if right_end < focus_end:
+                return False
+
+            # check that the next next object starts after we end
+
+            try:
+                next_next = self.timeline.getNextTimelineObject(right_obj)
+                if next_next.start < focus_end:
+                    return False
+            except TimelineError:
+                pass
+
+        return True
+
+    def finish(self):
+
+        if isinstance(self.focus, TrackObject):
+            focus_timeline_object = self.focus.timeline_object
+        else:
+            focus_timeline_object = self.focus
+        initial_position = self.default_originals[focus_timeline_object][0]
+        initial_priority = self.default_originals[focus_timeline_object][-1]
+
+        final_priority = self.focus.priority
+        final_position = self.focus.start
+
+        priority = final_priority
+
+        # special case for transitions. Allow a single object to overlap
+        # either of its two neighbors if it overlaps no other objects
+        if len(self.timeline_objects) == 1:
+            if not self._overlapsAreTransitions(focus_timeline_object,
+                priority):
+                self._defaultTo (initial_position, initial_priority)
+            EditingContext.finish(self)
+            return
+
+        # adjust priority
+        overlap = False
+        while True:
+            left_gap, right_gap = self._getGapsAtPriority(priority)
+
+            if left_gap is invalid_gap or right_gap is invalid_gap:
+                overlap = True
+
+                if priority == initial_priority:
+                    break
+
+                if priority > initial_priority:
+                    priority -= 1
+                else:
+                    priority += 1
+
+
+                self._defaultTo(final_position, priority)
+            else:
+                overlap = False
+                break
+
+        if not overlap:
+            EditingContext.finish(self)
+            return
+
+        self._defaultTo(initial_position, priority)
+        delta = final_position - initial_position
+        left_gap, right_gap = self._getGapsAtPriority(priority)
+
+        if delta > 0 and right_gap.duration < delta:
+            final_position = initial_position + right_gap.duration
+        elif delta < 0 and left_gap.duration < abs(delta):
+            final_position = initial_position - left_gap.duration
+
+        self._defaultTo(final_position, priority)
+        EditingContext.finish(self)
+
     def _defaultTo(self, position, priority):
         if self._snap:
             position = self.timeline.snapToEdge(position,
                 position + self.default_span)
 
         priority = max(self.min_priority, priority)
-        left_gap, right_gap = self._getGapsAtPriority(priority)
-
-        if left_gap is invalid_gap or right_gap is invalid_gap:
-            if priority == self._last_priority:
-                # abort move
-                return self._last_position, self._last_priority
-
-            # try to do the same time move, using the current priority
-            return self._defaultTo(position, self._last_priority)
-
-        delta = position - self.focus.start
-        if delta > 0 and right_gap.duration < delta:
-            position = self.focus.start + right_gap.duration
-        elif delta < 0 and left_gap.duration < abs(delta):
-            position = self.focus.start - left_gap.duration
+        position = max(self.min_position, position)
 
         self.focus.priority = priority
         self.focus.setStart(position, snap = self._snap)
@@ -1181,6 +1307,27 @@ class TrimStartContext(EditingContext):
         EditingContext.__init__(self, timeline, focus, other)
         self.adjacent = timeline.edges.getObjsAdjacentToStart(focus)
         self.adjacent_originals = self._saveValues(self.adjacent)
+        self.tracks = set([])
+        if isinstance(self.focus, TrackObject):
+            focus_timeline_object = self.focus.timeline_object
+            self.tracks.add(self.focus.track)
+        else:
+            focus_timeline_object = self.focus
+            tracks = set(track_object.track for track_object in
+                    focus.track_objects)
+            self.tracks.update(tracks)
+        self.focus_timeline_object = focus_timeline_object
+        self.default_originals = self._saveValues([focus_timeline_object])
+
+        ripple = self.timeline.getObjsBeforeTime(focus.start)
+        assert not focus.timeline_object in ripple or focus.duration == 0
+        self.ripple_originals = self._saveValues(ripple)
+        self.ripple_offsets = self._getOffsets(focus.start, focus.priority,
+            ripple)
+        if ripple:
+            self.ripple_min = focus.start - min((obj.start for obj in ripple))
+        else:
+            self.ripple_min = 0
 
     def _rollTo(self, position, priority):
         earliest = self.focus.start - self.focus.in_point
@@ -1188,22 +1335,73 @@ class TrimStartContext(EditingContext):
         for obj in self.adjacent:
             duration = max(0, position - obj.start)
             obj.setDuration(duration, snap=False)
+        return position, priority
 
     def _finishRoll(self):
         self._restoreValues(self.adjacent_originals)
 
-    def _defaultTo(self, position, priority):
+    def _rippleTo(self, position, priority):
         earliest = self.focus.start - self.focus.in_point
+        latest = earliest + self.focus.factory.duration
+
+        if self.snap:
+            position = self.timeline.snapToEdge(position)
+
+        position = min(latest, max(position, earliest))
+        self.focus.trimStart(position)
+        r_position = max(position, self.ripple_min)
+        for obj, (s_offset, p_offset) in self.ripple_offsets.iteritems():
+            obj.setStart(r_position + s_offset)
+
+        return position, priority
+
+    def _finishRipple(self):
+        self._restoreValues(self.ripple_originals)
+
+    def _defaultTo(self, position, priority):
+        earliest = max(0, self.focus.start - self.focus.in_point)
         self.focus.trimStart(max(position, earliest), snap=self.snap)
 
         return position, priority
 
+    def finish(self):
+        initial_position = self.default_originals[self.focus_timeline_object][0]
+
+        timeline_objects = [self.focus_timeline_object]
+        left_gap, right_gap = self._getGapsAtPriority(self.focus.priority,
+                timeline_objects, self.tracks)
+
+        if left_gap is invalid_gap:
+            self._defaultTo(initial_position, self.focus.priority)
+            left_gap, right_gap = Gap.findAroundObject(self.focus_timeline_object)
+            position = initial_position - left_gap.duration
+            self._defaultTo(position, self.focus.priority)
+        EditingContext.finish(self)
+
 class TrimEndContext(EditingContext):
 
     def __init__(self, timeline, focus, other):
         EditingContext.__init__(self, timeline, focus, other)
         self.adjacent = timeline.edges.getObjsAdjacentToEnd(focus)
         self.adjacent_originals = self._saveValues(self.adjacent)
+        self.tracks = set([])
+        if isinstance(self.focus, TrackObject):
+            focus_timeline_object = self.focus.timeline_object
+            self.tracks.add(focus.track)
+        else:
+            focus_timeline_object = self.focus
+            tracks = set(track_object.track for track_object in
+                    focus.track_objects)
+            self.tracks.update(tracks)
+        self.focus_timeline_object = focus_timeline_object
+        self.default_originals = self._saveValues([focus_timeline_object])
+
+        reference = focus.start + focus.duration
+        ripple = self.timeline.getObjsAfterTime(reference)
+
+        self.ripple_originals = self._saveValues(ripple)
+        self.ripple_offsets = self._getOffsets(reference, self.focus.priority,
+            ripple)
 
     def _rollTo(self, position, priority):
         if self._snap:
@@ -1212,16 +1410,50 @@ class TrimEndContext(EditingContext):
         self.focus.setDuration(duration)
         for obj in self.adjacent:
             obj.trimStart(position)
+        return position, priority
 
     def _finishRoll(self):
         self._restoreValues(self.adjacent_originals)
 
+    def _rippleTo(self, position, priority):
+        earliest = self.focus.start - self.focus.in_point
+        latest = earliest + self.focus.factory.duration
+        if self.snap:
+            position = self.timeline.snapToEdge(position)
+        position = min(latest, max(position, earliest))
+        duration = position - self.focus.start
+        self.focus.setDuration(duration)
+        for obj, (s_offset, p_offset) in self.ripple_offsets.iteritems():
+            obj.setStart(position + s_offset)
+
+        return position, priority
+
+    def _finishRipple(self):
+        self._restoreValues(self.ripple_originals)
+
     def _defaultTo(self, position, priority):
         duration = max(0, position - self.focus.start)
         self.focus.setDuration(duration, snap=self.snap)
 
         return position, priority
 
+    def finish(self):
+        EditingContext.finish(self)
+
+        initial_position, initial_duration = \
+                self.default_originals[self.focus_timeline_object][0:2]
+        absolute_initial_duration = initial_position + initial_duration
+
+        timeline_objects = [self.focus_timeline_object]
+        left_gap, right_gap = self._getGapsAtPriority(self.focus.priority,
+                timeline_objects, self.tracks)
+
+        if right_gap is invalid_gap:
+            self._defaultTo(absolute_initial_duration, self.focus.priority)
+            left_gap, right_gap = Gap.findAroundObject(self.focus_timeline_object)
+            duration = absolute_initial_duration + right_gap.duration
+            self._defaultTo(duration, self.focus.priority)
+
 class Timeline(Signallable, Loggable):
     """
     Top-level container for L{TimelineObject}s.
@@ -1293,7 +1525,7 @@ class Timeline(Signallable, Loggable):
         try:
             self.tracks.remove(track)
         except ValueError:
-            raise TimelineError()
+            raise TimelineError("Track not controlled by this Timeline")
 
         if removeTrackObjects:
             track.removeAllTrackObjects()
@@ -1325,12 +1557,12 @@ class Timeline(Signallable, Loggable):
         """
         self.debug("obj:%r", obj)
         if obj.timeline is not None:
-            raise TimelineError()
+            raise TimelineError("TimelineObject already controlled by another Timeline")
 
         # FIXME : wait... what's wrong with having empty timeline objects ??
         # And even if it was.. this shouldn't be checked here imho.
         if not obj.track_objects:
-            raise TimelineError()
+            raise TimelineError("TimelineObject doesn't have any TrackObject (THIS IS A VERY DUBIOUS CHECK, WE SHOULD ACCEPT THIS)")
 
         self._connectToTimelineObject(obj)
 
@@ -1354,7 +1586,7 @@ class Timeline(Signallable, Loggable):
         try:
             self.timeline_objects.remove(obj)
         except ValueError:
-            raise TimelineError()
+            raise TimelineError("TimelineObject not controlled by this Timeline")
 
         if obj.link is not None:
             obj.link.removeTimelineObject(obj)
@@ -1418,7 +1650,7 @@ class Timeline(Signallable, Loggable):
         self.debug("factory:%r", factory)
         output_streams = factory.getOutputStreams()
         if not output_streams:
-            raise TimelineError()
+            raise TimelineError("SourceFactory doesn't provide any Output Streams")
 
         if stream_map is None:
             stream_map = self._getSourceFactoryStreamMap(factory)
@@ -1428,7 +1660,7 @@ class Timeline(Signallable, Loggable):
                 self.error("Couldn't find a complete stream mapping (self:%d < factory:%d)",
                            len(stream_map), len(output_streams))
                 if strict:
-                    raise TimelineError()
+                    raise TimelineError("Couldn't map all streams to available Tracks")
 
         timeline_object = TimelineObject(factory)
         start = 0
@@ -1460,26 +1692,61 @@ class Timeline(Signallable, Loggable):
 
         return output_stream_to_track_map
 
-    def getPreviousTimelineObject(self, obj, priority=-1):
-        prev = getPreviousObject(obj, self.timeline_objects, priority)
+    def getPreviousTimelineObject(self, obj, priority=-1, tracks=None):
+        if tracks is None:
+            skip = None
+        else:
+            def skipIfNotInTheseTracks(timeline_object):
+                return self._skipIfNotInTracks(timeline_object, tracks)
+            skip = skipIfNotInTheseTracks
+
+        prev = getPreviousObject(obj, self.timeline_objects,
+                priority, skip=skip)
+
         if prev is None:
             raise TimelineError("no previous timeline object", obj)
 
         return prev
 
-    def getNextTimelineObject(self, obj, priority=-1):
-        next = getNextObject(obj, self.timeline_objects, priority)
+    def getNextTimelineObject(self, obj, priority=-1, tracks=None):
+        if tracks is None:
+            skip = None
+        else:
+            def skipIfNotInTheseTracks(timeline_object):
+                return self._skipIfNotInTracks(timeline_object, tracks)
+            skip = skipIfNotInTheseTracks
+
+        next = getNextObject(obj, self.timeline_objects, priority, skip)
         if next is None:
             raise TimelineError("no next timeline object", obj)
 
         return next
 
+    def _skipIfNotInTracks(self, timeline_object, tracks):
+        timeline_object_tracks = set(track_object.track for track_object in
+                timeline_object.track_objects)
+
+        return not tracks.intersection(timeline_object_tracks)
+
     def setSelectionToObj(self, obj, mode):
         """
         Update the timeline's selection with the given object and mode.
 
         @see: L{Selection.setToObj}
         """
+        if mode == SELECT_BETWEEN:
+            if self.selection.last_single_obj:
+                last = self.selection.last_single_obj
+                earliest = min(last.start, obj.start)
+                latest = max(last.start + last.duration,
+                    obj.start + obj.duration)
+                min_priority = min(last.priority, obj.priority)
+                max_priority = max(last.priority, obj.priority)
+                objs = self.getObjsInRegion(earliest, latest,
+                    min_priority, max_priority)
+                self.setSelectionTo(objs, SELECT)
+                return
+
         self.selection.setToObj(obj, mode)
 
     def setSelectionTo(self, selection, mode):
@@ -1549,9 +1816,6 @@ class Timeline(Signallable, Loggable):
                 tracks.append(track_object.track)
                 new_timeline_object.addTrackObject(new_track_object)
 
-        for i, track_object in enumerate(new_timeline_object.track_objects):
-            tracks[i].addTrackObject(track_object)
-
         self.addTimelineObject(new_timeline_object)
 
         old_track_objects = []
@@ -1571,13 +1835,13 @@ class Timeline(Signallable, Loggable):
             self.selection.setTo(timeline_object.track_objects, UNSELECT)
 
             for track_object in list(timeline_object.track_objects):
-                timeline_object.removeTrackObject(track_object)
-                new_timeline_object = TimelineObject(track_object.factory)
-                new_timeline_object.addTrackObject(track_object)
+                new_track_object = track_object.copy()
+                new_timeline_object = TimelineObject(new_track_object.factory)
+                new_timeline_object.addTrackObject(new_track_object)
                 self.addTimelineObject(new_timeline_object)
                 new_track_objects.extend(new_timeline_object.track_objects)
 
-            self.removeTimelineObject(timeline_object)
+            self.removeTimelineObject(timeline_object, deep=True)
 
         self.selection.setTo(new_track_objects, SELECT_ADD)
 
@@ -1590,6 +1854,17 @@ class Timeline(Signallable, Loggable):
             self.removeTimelineObject(timeline_object, deep=True)
         self.selection.setTo(set([]), SELECT)
 
+    def split(self, time):
+        """
+        Splits objects under the playehad. If the selection is not empty, the
+        split only applies to selected clips. Otherwise it applies to all
+        clips"""
+        objs = set(self.getObjsAtTime(time))
+        if len(self.selection):
+            objs = self.selection.selected.intersection(objs)
+        for obj in objs:
+            obj.split(time)
+
     def rebuildEdges(self):
         self.edges = TimelineEdges()
         for timeline_object in self.timeline_objects:
@@ -1634,6 +1909,12 @@ class Timeline(Signallable, Loggable):
 
         self.emit("disable-updates", False)
 
+    def getObjsAtTime(self, time_):
+        objects = self.timeline_objects
+        return [obj for obj in objects if
+            (obj.start < time_) and 
+            ((obj.start + obj.duration) > time_)]
+
     def getObjsAfterObj(self, obj):
         return self.getObjsAfterTime(obj.start + obj.duration)
 
@@ -1647,3 +1928,15 @@ class Timeline(Signallable, Loggable):
     def getObjsBeforeTime(self, target):
         return [to for to in self.timeline_objects 
             if to.start + to.duration <=target]
+
+    def getObjsInRegion(self, start, end, min_priority=0,
+        max_priority=4294967295L):
+        objs = []
+        for to in self.timeline_objects:
+            if ((to.start >= start) and
+                ((to.start + to.duration) <= end) and
+                (to.priority >= min_priority) and
+                (to.priority <= max_priority)):
+                objs.append(to)
+        return objs
+
