@@ -40,10 +40,13 @@ from zoominterface import Zoomable
 from pitivi.ui.common import LAYER_HEIGHT_EXPANDED, LAYER_SPACING
 from pitivi.timeline.timeline import MoveContext
 from pitivi.utils import Seeker
+from pitivi.ui.filelisterrordialog import FileListErrorDialog
+from pitivi.ui.curve import Curve
 
 # tooltip text for toolbar
 DELETE = _("Delete Selected")
-RAZOR = _("Cut clip at mouse position")
+SPLIT = _("Split clip at playhead position")
+KEYFRAME = _("Create a keyframe")
 ZOOM_IN =  _("Zoom In")
 ZOOM_OUT =  _("Zoom Out")
 UNLINK = _("Break links between clips")
@@ -64,7 +67,8 @@ ui = '''
         </menu>
         <menu action="Timeline">
             <placeholder name="Timeline">
-                <menuitem action="Razor" />
+                <menuitem action="Split" />
+                <menuitem action="Keyframe" />
                 <separator />
                 <menuitem action="DeleteObj" />
                 <menuitem action="LinkObj" />
@@ -76,10 +80,9 @@ ui = '''
     </menubar>
     <toolbar name="TimelineToolBar">
         <placeholder name="Timeline">
-            <toolitem action="ZoomOut" />
-            <toolitem action="ZoomIn" />
             <separator />
-            <toolitem action="Razor" />
+            <toolitem action="Split" />
+            <toolitem action="Keyframe" />
             <separator />
             <toolitem action="DeleteObj" />
             <toolitem action="UnlinkObj" />
@@ -89,6 +92,9 @@ ui = '''
         </placeholder>
     </toolbar>
     <accelerator action="DeleteObj" />
+    <accelerator action="ControlEqualAccel" />
+    <accelerator action="ControlKPAddAccel" />
+    <accelerator action="ControlKPSubtractAccel" />
 </ui>
 '''
 
@@ -111,6 +117,73 @@ ui = '''
 #    |
 #    +--Status Bar ??
 
+class InfoStub(gtk.HBox, Loggable):
+    """
+    Box used to display information on the current state of the timeline
+    """
+
+    def __init__(self):
+        gtk.HBox.__init__(self)
+        Loggable.__init__(self)
+        self.errors = []
+        self.showing = False
+        self._errorsmessage = _("One or more GStreamer errors has occured!")
+        self._makeUI()
+
+    def _makeUI(self):
+        self.set_spacing(6)
+        self.erroricon = gtk.image_new_from_stock(gtk.STOCK_DIALOG_WARNING,
+                                                  gtk.ICON_SIZE_SMALL_TOOLBAR)
+
+        self.pack_start(self.erroricon, expand=False)
+
+
+        self.infolabel = gtk.Label(self._errorsmessage)
+        self.infolabel.set_alignment(0, 0.5)
+
+        self.questionbutton = gtk.Button()
+        self.questionbutton.set_image(gtk.image_new_from_stock(gtk.STOCK_INFO,
+                                                               gtk.ICON_SIZE_SMALL_TOOLBAR))
+        self.questionbutton.connect("clicked", self._questionButtonClickedCb)
+        self._questionshowing = False
+
+        self.pack_start(self.infolabel, expand=True, fill=True)
+        self.pack_start(self.questionbutton, expand=False)
+
+    def addErrors(self, *args):
+        self.errors.append(args)
+        self.show()
+
+    def _errorDialogBoxCloseCb(self, dialog):
+        dialog.destroy()
+
+    def _errorDialogBoxResponseCb(self, dialog, unused_response):
+        dialog.destroy()
+
+    def _questionButtonClickedCb(self, unused_button):
+        msgs = (_("Error List"),
+            _("The following errors have been reported:"))
+        # show error dialog
+        dbox = FileListErrorDialog(*msgs)
+        dbox.connect("close", self._errorDialogBoxCloseCb)
+        dbox.connect("response", self._errorDialogBoxResponseCb)
+        for reason, extra in self.errors:
+            dbox.addFailedFile(None, reason, extra)
+        dbox.show()
+        # reset error list
+        self.errors = []
+        self.hide()
+
+    def show(self):
+        self.log("showing")
+        self.show_all()
+        self.showing = True
+
+    def hide(self):
+        self.log("hiding")
+        gtk.VBox.hide(self)
+        self.showing = False
+
 class Timeline(gtk.Table, Loggable, Zoomable):
 
     # the screen width of the current unit
@@ -125,6 +198,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         Zoomable.__init__(self)
         self.log("Creating Timeline")
 
+        self._updateZoom = True
         self.project = None
         self.ui_manager = ui_manager
         self.app = instance
@@ -137,52 +211,85 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._prev_duration = 0
         self.shrink = True
         self.rate = gst.Fraction(1,1)
-        self._seeker = Seeker(80)
-        self._seeker.connect('seek', self._seekerSeekCb)
 
     def _createUI(self):
         self.leftSizeGroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+        self.props.row_spacing = 2
+        self.props.column_spacing = 2
         self.hadj = gtk.Adjustment()
         self.vadj = gtk.Adjustment()
 
+        # zooming slider
+        self._zoomAdjustment = gtk.Adjustment()
+        self._zoomAdjustment.set_value(Zoomable.getCurrentZoomLevel())
+        self._zoomAdjustment.connect("value-changed",
+            self._zoomAdjustmentChangedCb)
+        self._zoomAdjustment.props.lower = 0
+        self._zoomAdjustment.props.upper = Zoomable.zoom_steps
+        zoomslider = gtk.HScale(self._zoomAdjustment)
+        zoomslider.props.draw_value = False
+        zoomslider.set_tooltip_text(_("Zoom Timeline"))
+        self.attach(zoomslider, 0, 1, 0, 1, yoptions=0, xoptions=gtk.FILL)
+
         # controls for tracks and layers
         self._controls = TimelineControls()
-        controlwindow = gtk.ScrolledWindow(None, self.vadj)
-        controlwindow.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
-        controlwindow.add_with_viewport(self._controls)
+        controlwindow = gtk.Viewport(None, self.vadj)
+        controlwindow.add(self._controls)
         controlwindow.set_size_request(-1, 1)
+        controlwindow.set_shadow_type(gtk.SHADOW_OUT)
         self.attach(controlwindow, 0, 1, 1, 2, xoptions=0)
 
         # timeline ruler
-        self.ruler = ruler.ScaleRuler(self.hadj)
+        self.ruler = ruler.ScaleRuler(self.app, self.hadj)
         self.ruler.set_size_request(0, 25)
         self.ruler.set_border_width(2)
         self.ruler.connect("key-press-event", self._keyPressEventCb)
-        self.attach(self.ruler, 1, 2, 0, 1, yoptions=0)
+        self.ruler.connect("size-allocate", self._rulerSizeAllocateCb)
+        rulerframe = gtk.Frame()
+        rulerframe.set_shadow_type(gtk.SHADOW_OUT)
+        rulerframe.add(self.ruler)
+        self.attach(rulerframe, 1, 2, 0, 1, yoptions=0)
 
         # proportional timeline
         self._canvas = TimelineCanvas(self.app)
         timelinewindow = gtk.ScrolledWindow(self.hadj, self.vadj)
         timelinewindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         timelinewindow.add(self._canvas)
-        #FIXME: remove padding between scrollbar and scrolled window
+        timelinewindow.set_shadow_type(gtk.SHADOW_IN)
+        timelinewindow.set_name("timelinewindow")
+
+        # temp fix for padding between scrollbar and scrolled window
+        #FIXME: should be set at an global position for easy editing?
+        gtk.rc_parse_string("""
+            style 'timelinewindow'
+            {
+                GtkScrolledWindow::scrollbar-spacing = 0
+            }
+            widget '*.timelinewindow' style 'timelinewindow'
+        """)
+
         self.attach(timelinewindow, 1, 2, 1, 2)
 
-        # drag and drop
-        self.drag_dest_set(gtk.DEST_DEFAULT_MOTION,
-            [dnd.FILESOURCE_TUPLE],
-            gtk.gdk.ACTION_COPY)
+        # error infostub
+        self.infostub = InfoStub()
+        self.attach(self.infostub, 1, 2, 2, 3, yoptions=0)
 
-        self.connect("drag-data-received", self._dragDataReceivedCb)
-        self.connect("drag-leave", self._dragLeaveCb)
-        self.connect("drag-drop", self._dragDropCb)
-        self.connect("drag-motion", self._dragMotionCb)
+        self.show_all()
+        self.infostub.hide()
 
         # toolbar actions
         actions = (
-            ("ZoomIn", gtk.STOCK_ZOOM_IN, None, None, ZOOM_IN,
+            ("ZoomIn", gtk.STOCK_ZOOM_IN, None, "<Control>plus", ZOOM_IN,
                 self._zoomInCb),
-            ("ZoomOut", gtk.STOCK_ZOOM_OUT, None, None, ZOOM_OUT,
+            ("ZoomOut", gtk.STOCK_ZOOM_OUT, None, "<Control>minus", ZOOM_OUT,
+                self._zoomOutCb),
+
+            # actions for adding additional accelerators
+            ("ControlEqualAccel", gtk.STOCK_ZOOM_IN, None, "<Control>equal", ZOOM_IN,
+                self._zoomInCb),
+            ("ControlKPAddAccel", gtk.STOCK_ZOOM_IN, None, "<Control>KP_Add", ZOOM_IN,
+                self._zoomInCb),
+            ("ControlKPSubtractAccel", gtk.STOCK_ZOOM_OUT, None, "<Control>KP_Subtract", ZOOM_OUT,
                 self._zoomOutCb),
         )
 
@@ -199,23 +306,27 @@ class Timeline(gtk.Table, Loggable, Zoomable):
                 self.groupSelected),
         )
 
-        toggle_actions = (
-            ("Razor", "pitivi-split", _("Razor"), "<Ctrl>R", RAZOR,
-                self.toggleRazor),
+        playhead_actions = (
+            ("Split", "pitivi-split", _("Split"), "S", SPLIT,
+                self.split),
+            ("Keyframe", "pitivi-keyframe", _("Keyframe"), "K", KEYFRAME,
+                self.keyframe),
         )
 
         actiongroup = gtk.ActionGroup("timelinepermanent")
         actiongroup.add_actions(actions)
-        actiongroup.add_toggle_actions(toggle_actions)
         self.ui_manager.insert_action_group(actiongroup, 0)
 
         actiongroup = gtk.ActionGroup("timelineselection")
         actiongroup.add_actions(selection_actions)
+        actiongroup.add_actions(playhead_actions)
         self.link_action = actiongroup.get_action("LinkObj")
         self.unlink_action = actiongroup.get_action("UnlinkObj")
         self.group_action = actiongroup.get_action("GroupObj")
         self.ungroup_action = actiongroup.get_action("UngroupObj")
         self.delete_action = actiongroup.get_action("DeleteObj")
+        self.split_action = actiongroup.get_action("Split")
+        self.keyframe_action = actiongroup.get_action("Keyframe")
 
         self.ui_manager.insert_action_group(actiongroup, -1)
 
@@ -271,9 +382,6 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             pipeline.getDuration()))
         self._seeker.seek(seekvalue)
 
-    def _seekerSeekCb(self, seeker, position, format):
-        self.project.pipeline.seek(position, format)
-
     def _buttonPress(self, window, event):
         self.shrink = False
 
@@ -325,6 +433,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         context.drop_finish(True, timestamp)
         self._factories = None
         self._temp_objects = None
+        self.app.current.seeker.seek(self._position)
         return True
 
     def _dragDataReceivedCb(self, unused_layout, context, x, y,
@@ -338,10 +447,11 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         # tell current project to import the uri
         # wait for source-added signal, meanwhile ignore dragMotion signals
         # when ready, add factories to the timeline.
-        if targetType == dnd.TYPE_PITIVI_FILESOURCE:
-            uris = selection.data.split("\n")
-        else:
+        if targetType != dnd.TYPE_PITIVI_FILESOURCE:
             context.finish(False, False, timestamp)
+            return
+
+        uris = selection.data.split("\n")
         self._factories = [self.project.sources.getUri(uri) for uri in uris]
         context.drag_status(gtk.gdk.ACTION_COPY, timestamp)
         return True
@@ -360,15 +470,23 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
 ## Zooming and Scrolling
 
+    def _zoomAdjustmentChangedCb(self, adjustment):
+        # GTK crack
+        self._updateZoom = False
+        Zoomable.setZoomLevel(int(adjustment.get_value()))
+        self._updateZoom = True
+
     def zoomChanged(self):
-        # this has to be in a timeout, because the resize hasn't actually
-        # completed yet, and so the canvas can't actually complete the scroll
-        gobject.idle_add(self.scrollToPlayhead)
+        self._canvas.props.redraw_when_scrolled = True
+        if self._updateZoom:
+            self._zoomAdjustment.set_value(self.getCurrentZoomLevel())
+        self.ruler.queue_resize()
+        self.ruler.queue_draw()
 
     def timelinePositionChanged(self, position):
         self._position = position
         self.ruler.timelinePositionChanged(position)
-        self._canvas._position = position
+        self._canvas.timelinePositionChanged(position)
         if self._state == gst.STATE_PLAYING:
             self.scrollToPlayhead()
 
@@ -376,11 +494,16 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self._state = state
 
     def scrollToPlayhead(self):
-        width = self.get_allocation().width
+        """
+        Scroll the current position as close to the center of the view
+        as possible (as close as the timeline canvas allows).
+        """
+        page_size = self.hadj.get_page_size()
+
         new_pos = Zoomable.nsToPixel(self._position)
         scroll_pos = self.hadj.get_value()
-        if (new_pos < scroll_pos) or (new_pos > scroll_pos + width):
-            self.scrollToPosition(new_pos - width / 2)
+        if (new_pos > scroll_pos + page_size) or (new_pos < scroll_pos):
+            self.scrollToPosition(min(new_pos - page_size / 2, self.hadj.upper - page_size - 1))
         return False
 
     def scrollToPosition(self, position):
@@ -395,6 +518,9 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         self.hadj.set_value(position)
         return False
 
+    def _rulerSizeAllocateCb(self, ruler, allocation):
+        self._canvas.props.redraw_when_scrolled = False
+
 ## Project callbacks
 
     def _setProject(self):
@@ -405,13 +531,14 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             self._canvas.zoomChanged()
             self.ruler.setProjectFrameRate(self.project.getSettings().videorate)
             self.ruler.zoomChanged()
-            self._settingsChangedCb(self.project)
+            self._settingsChangedCb(self.project, None, self.project.getSettings())
+            self._seeker = self.project.seeker
 
     project = receiver(_setProject)
 
     @handler(project, "settings-changed")
-    def _settingsChangedCb(self, project):
-        rate = self.project.getSettings().videorate
+    def _settingsChangedCb(self, project, old, new):
+        rate = new.videorate
         self.rate = float(1 / rate)
         self.ruler.setProjectFrameRate(rate)
 
@@ -449,6 +576,8 @@ class Timeline(gtk.Table, Loggable, Zoomable):
         unlink = False
         group = False
         ungroup = False
+        split = False
+        keyframe = False
         timeline_objects = {}
         if timeline.selection:
             delete = True
@@ -460,6 +589,7 @@ class Timeline(gtk.Table, Loggable, Zoomable):
             duration = None
             for obj in self.timeline.selection:
                 if obj.link:
+                    link = False
                     unlink = True
 
                 if len(obj.track_objects) > 1:
@@ -472,11 +602,17 @@ class Timeline(gtk.Table, Loggable, Zoomable):
                     start = obj.start
                     duration = obj.duration
 
+            split = True
+            keyframe = True
+
         self.delete_action.set_sensitive(delete)
         self.link_action.set_sensitive(link)
         self.unlink_action.set_sensitive(unlink)
         self.group_action.set_sensitive(group)
         self.ungroup_action.set_sensitive(ungroup)
+        self.split_action.set_sensitive(split)
+        self.keyframe_action.set_sensitive(keyframe)
+
 
 ## ToolBar callbacks
 
@@ -506,14 +642,40 @@ class Timeline(gtk.Table, Loggable, Zoomable):
 
     def ungroupSelected(self, unused_action):
         if self.timeline:
+            self.app.action_log.begin("ungroup")
             self.timeline.ungroupSelection()
+            self.app.action_log.commit()
 
     def groupSelected(self, unused_action):
         if self.timeline:
             self.timeline.groupSelection()
 
-    def toggleRazor(self, action):
-        if action.props.active:
-            self._canvas.activateRazor(action)
-        else:
-            self._canvas.deactivateRazor()
+    def split(self, action):
+        self.app.action_log.begin("split")
+        self.timeline.split(self._position)
+        self.app.action_log.commit()
+        # work-around for 603149
+        self.project.seeker.seek(self._position)
+
+    def keyframe(self, action):
+        timeline_position = self._position
+        selected = self.timeline.selection.getSelectedTrackObjs()
+        
+        for obj in selected:
+            keyframe_exists = False
+
+            position_in_obj = timeline_position - obj.start
+            interpolators = obj.getInterpolators()
+            for value in interpolators:
+                interpolator = obj.getInterpolator(value)
+                keyframes = interpolator.getInteriorKeyframes()
+                for kf in keyframes:
+                    if kf.getTime() == position_in_obj:
+                        keyframe_exists = True
+                        self.app.action_log.begin("remove volume point")
+                        interpolator.removeKeyframe(kf)
+                        self.app.action_log.commit()
+                if keyframe_exists == False:
+                    self.app.action_log.begin("add volume point")
+                    interpolator.newKeyframe(position_in_obj)
+                    self.app.action_log.commit()
