@@ -31,12 +31,15 @@ from pitivi.ui.point import Point
 from pitivi.ui.zoominterface import Zoomable
 from pitivi.settings import GlobalSettings
 from pitivi.ui.prefs import PreferencesDialog
-from pitivi.ui.common import TRACK_SPACING, unpack_cairo_pattern
+from pitivi.ui.common import TRACK_SPACING, unpack_cairo_pattern, \
+        LAYER_HEIGHT_EXPANDED, LAYER_SPACING
+from pitivi.ui.controller import Controller
+from pitivi.ui.curve import KW_LABEL_Y_OVERFLOW
 
 # cursors to be used for resizing objects
 ARROW = gtk.gdk.Cursor(gtk.gdk.ARROW)
 # TODO: replace this with custom cursor
-RAZOR_CURSOR = gtk.gdk.Cursor(gtk.gdk.XTERM)
+PLAYHEAD_CURSOR = gtk.gdk.Cursor(gtk.gdk.SB_H_DOUBLE_ARROW)
 
 GlobalSettings.addConfigOption('edgeSnapDeadband',
     section = "user-interface",
@@ -51,11 +54,25 @@ PreferencesDialog.addNumericPreference('edgeSnapDeadband',
         "operations"),
     lower = 0)
 
+class PlayheadController(Controller, Zoomable):
+
+    _cursor = PLAYHEAD_CURSOR
+
+
+    def __init__(self, *args, **kwargs):
+        Controller.__init__(self, *args, **kwargs)
+
+    def set_pos(self, item, pos):
+        self._canvas.app.current.seeker.seek(
+            Zoomable.pixelToNs(pos[0]))
+
 class TimelineCanvas(goocanvas.Canvas, Zoomable, Loggable):
 
+    __gtype_name__ = 'TimelineCanvas'
     __gsignals__ = {
-        "scroll-event":"override"
-        }
+        "scroll-event": "override",
+        "expose-event" : "override",
+    }
 
     _tracks = None
 
@@ -72,6 +89,8 @@ class TimelineCanvas(goocanvas.Canvas, Zoomable, Loggable):
         self._block_size_request = False
         self.props.integer_layout = True
         self.props.automatic_bounds = False
+        self.props.clear_background = False
+        self.get_root_item().set_simple_transform(0, 2.0, 1.0, 0)
 
         self._createUI()
         self.timeline = timeline
@@ -81,21 +100,28 @@ class TimelineCanvas(goocanvas.Canvas, Zoomable, Loggable):
         self._cursor = ARROW
         root = self.get_root_item()
         self.tracks = goocanvas.Group()
+        self.tracks.set_simple_transform(0, KW_LABEL_Y_OVERFLOW, 1.0, 0)
         root.add_child(self.tracks)
         self._marquee = goocanvas.Rect(
+            parent=root,
             stroke_pattern = unpack_cairo_pattern(0x33CCFF66),
             fill_pattern = unpack_cairo_pattern(0x33CCFF66),
             visibility = goocanvas.ITEM_INVISIBLE)
-        self._razor = goocanvas.Rect(
-            line_width=0,
-            fill_color="orange",
-            width=1,
-            visibility=goocanvas.ITEM_INVISIBLE)
-        root.add_child(self._marquee)
-        root.add_child(self._razor)
+        self._playhead = goocanvas.Rect(
+            y = -10,
+            parent=root,
+            line_width=1,
+            fill_color_rgba=0x000000FF,
+            stroke_color_rgba=0xFFFFFFFF,
+            width=3)
+        self._playhead_controller = PlayheadController(self._playhead)
         root.connect("motion-notify-event", self._selectionDrag)
         root.connect("button-press-event", self._selectionStart)
         root.connect("button-release-event", self._selectionEnd)
+        height = (LAYER_HEIGHT_EXPANDED + TRACK_SPACING + LAYER_SPACING) * 2
+        # add some padding for the horizontal scrollbar
+        height += 21
+        self.set_size_request(-1, height)
 
     def from_event(self, event):
         return Point(*self.convert_from_pixels(event.x, event.y))
@@ -139,11 +165,41 @@ class TimelineCanvas(goocanvas.Canvas, Zoomable, Loggable):
         event.window.set_cursor(self._cursor)
         return True
 
+    def do_expose_event(self, event):
+        allocation = self.get_allocation()
+        width = allocation.width
+        height = allocation.height
+        # draw the canvas background
+        # we must have props.clear_background set to False
+
+        self.style.apply_default_background(event.window,
+            True,
+            gtk.STATE_ACTIVE,
+            event.area,
+            event.area.x, event.area.y,
+            event.area.width, event.area.height)
+
+        y = 0
+        for track in self._tracks[:-1]:
+            y += track.height
+            self.style.paint_box(event.window,
+                gtk.STATE_NORMAL,
+                gtk.SHADOW_OUT,
+                event.area,
+                self,
+                "",
+                event.area.x - 5, y + 1,
+                event.area.width + 10,  TRACK_SPACING - 2)
+            y += TRACK_SPACING 
+
+        goocanvas.Canvas.do_expose_event(self, event)
+
 ## implements selection marquee
 
     _selecting = False
     _mousedown = None
     _marquee = None
+    _got_motion_notify = False
 
     def _normalize(self, p1, p2):
         w, h = p2 - p1
@@ -159,6 +215,7 @@ class TimelineCanvas(goocanvas.Canvas, Zoomable, Loggable):
 
     def _selectionDrag(self, item, target, event):
         if self._selecting:
+            self._got_motion_notify = True
             cur = self.from_event(event)
             pos, size = self._normalize(self._mousedown, cur)
             m = self._marquee
@@ -178,15 +235,21 @@ class TimelineCanvas(goocanvas.Canvas, Zoomable, Loggable):
         return True
 
     def _selectionEnd(self, item, target, event):
+        seeker = self.app.current.seeker
         self.pointer_ungrab(self.get_root_item(), event.time)
         self._selecting = False
         self._marquee.props.visibility = goocanvas.ITEM_INVISIBLE
-        mode = 0
-        if event.get_state() & gtk.gdk.SHIFT_MASK:
-            mode = 1
-        if event.get_state() & gtk.gdk.CONTROL_MASK:
-            mode = 2
-        self.timeline.setSelectionTo(self._objectsUnderMarquee(), mode)
+        if not self._got_motion_notify:
+            self.timeline.setSelectionTo(set(), 0)
+            seeker.seek(Zoomable.pixelToNs(event.x))
+        else:
+            self._got_motion_notify = False
+            mode = 0
+            if event.get_state() & gtk.gdk.SHIFT_MASK:
+                mode = 1
+            if event.get_state() & gtk.gdk.CONTROL_MASK:
+                mode = 2
+            self.timeline.setSelectionTo(self._objectsUnderMarquee(), mode)
         return True
 
     def _objectsUnderMarquee(self):
@@ -197,58 +260,13 @@ class TimelineCanvas(goocanvas.Canvas, Zoomable, Loggable):
                 TrackObject)))
         return set()
 
-## Razor Tool Implementation
+## playhead implementation
 
-    def activateRazor(self, action):
-        self._razor_sigid = self.connect("button_press_event",
-            self._razorClickedCb)
-        self._razor_release_sigid = self.connect("button_release_event",
-            self._razorReleasedCb)
-        self._razor_motion_sigid = self.connect("motion_notify_event",
-            self._razorMovedCb)
-        self._razor.props.visibility = goocanvas.ITEM_VISIBLE
-        self._action = action
-        return True
+    position = 0
 
-    def deactivateRazor(self):
-        self.disconnect(self._razor_sigid)
-        self.disconnect(self._razor_motion_sigid)
-        self.disconnect(self._razor_release_sigid)
-        self._razor.props.visibility = goocanvas.ITEM_INVISIBLE
-
-    def _razorMovedCb(self, canvas, event):
-        def snap(x):
-            pos = self.nsToPixel(self._position)
-            if abs(x - pos) <= self.settings.edgeSnapDeadband:
-                return pos
-            return x
-        x, y = self.convert_from_pixels(event.x, event.y)
-        self._razor.props.x = snap(self.nsToPixel(self.pixelToNs(x)))
-        return True
-
-    def _razorReleasedCb(self, unused_canvas, event):
-        self._action.props.active = False
-
-        x, y = self.convert_from_pixels(event.x, event.y)
-        bounds = goocanvas.Bounds(x, y, x, y)
-        items = self.get_items_in_area(bounds, True, True, True)
-        if items:
-            for item in items:
-                if isinstance(item, TrackObject):
-                    self.app.action_log.begin("split object")
-                    item.element.split(self._snapToPlayhead(self.pixelToNs(x)))
-                    self.app.action_log.commit()
-
-        return True
-
-    def _razorClickedCb(self, unused_canvas, unused_event):
-        return True
-
-    def _snapToPlayhead(self, time):
-        thresh = self.pixelToNs(self.settings.edgeSnapDeadband)
-        if abs(time - self._position) <= thresh:
-            return self._position
-        return time
+    def timelinePositionChanged(self, position):
+        self.position = position
+        self._playhead.props.x = self.nsToPixel(position)
 
     max_duration = 0
 
@@ -257,19 +275,18 @@ class TimelineCanvas(goocanvas.Canvas, Zoomable, Loggable):
         self._request_size()
 
     def _request_size(self):
+        alloc = self.get_allocation()
         w = Zoomable.nsToPixel(self.max_duration)
-        self.set_bounds(0, 0, w, self._height)
-        self._razor.props.height = self._height
-        self.get_root_item().changed(True)
-
-## Zoomable Override
+        h = max(self._height, alloc.height)
+        self.set_bounds(0, 0, w, h)
+        self._playhead.props.height = h + 10
 
     def zoomChanged(self):
         if self.timeline:
             self.timeline.dead_band = self.pixelToNs(
                 self.settings.edgeSnapDeadband)
             self._request_size()
-
+            self.timelinePositionChanged(self.position)
 
 ## settings callbacks
 
